/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS                       File "include/adc_stm32.h"           */
/*  Declaration of the procedures of the DACs'.                              */
/*                                                                           */
/*  Author : JYA                        Last modification : 11/23/2018       */
/*                                                                           */
/*  (c) Copyright Hyperpanel Lab, All Rights Reserved                        */
/*  IDDN.FR.001.320001.*.S.P.1990.000.10000                                  */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations ========================================================


         1  Defines
         2  Structures and typedefs
         3  Public variables
         4  Prototypes of public procedures


*/


#ifndef ADC_STM32_H                    /* If this file is not already        */
#define ADC_STM32_H                    /* included, include it.              */


#ifdef NEW_ADC

/*
    1    Defines
    ____________
*/


/*
    2    Structures and typedefs
    ____________________________
*/

typedef struct                         /* Parametres du device ADC           */
  BEGIN                                /* ---------------------------------- */
    UCHAR         adc_num            ; /* ADC number                         */
    CHAR         *tim_name           ; /* Timer name                         */
    UCHAR         dma_num            ; /* DMA number                         */
    UCHAR         trig               ; /* External trigger index             */
    UCHAR         dma_sdn            ; /* DMA sub-device number              */
  END             Adc_stm32_def      ; /* ---------------------------------- */

typedef struct                         /* Low level Context                  */
  BEGIN                                /* Constant values ------------------ */
    IWORD         dev                ; /* Device num. for the generic ADC drv*/
    UCHAR         adc_num            ; /* ADC number                         */
    UCHAR         tim_num            ; /* Timer number                       */
    UCHAR         dma_num            ; /* DMA number                         */
    UCHAR         dma_sdn            ; /* DMA sub-device number              */
    UCHAR         trig               ; /* External trigger index             */
    volatile Adc *adc_reg            ; /* ADC registers base address         */
                                       /*                                    */
                                       /* Almost constant values ----------- */
    UINT          smp_prd            ; /* Sampling period in number of       */
                                       /* machine cycles                     */
    UCHAR         dep                ; /* Sample depth                       */
    UCHAR         dma                ; /* 0: FIFO, 1: DMA                    */
                                       /*                                    */
                                       /* For DMA transfers ---------------- */
    UCHAR         cnv_in_prg         ; /* A to D conversion state            */
    UCHAR         half_it            ; /* 0: transfer complete interrupt     */
                                       /* 1: half transfer interrupt         */
    UCHAR        *loc_b_ptr          ; /* Pointer in local buffer used by DMA*/
    UCHAR        *app_b_ptr          ; /* Pointer in buffer sent by applicati*/
    UCHAR        *app_b_end_ad       ; /* End address of buffer sent by appli*/
                                       /*                                    */
                                       /* Error counter -------------------- */
    UINT          cnr_ovr            ; /* Overrun error counter              */
  END             Adc_stm32          ; /* ---------------------------------- */


/*
    3    Public variables
    _____________________
*/


//extern const Adc_desc adc_desc_stm32 ; /* ADC  low level interface           */
extern       Adc_stm32 adc_stm32[]   ; /* Context table of tim_stm32.c       */


/*
    4    Prototypes of public procedures
    ____________________________________
*/


/* Declaration of the procedures ........................................... */

IWORD handlerADC_stm32                 (IWORD,IWORD                          );


#else                                  /* NEW_ADC                            */


/*
    1    Defines
    ____________
*/


/*
    2    Structures and typedefs
    ____________________________
*/

typedef struct                         /* Parametres du device ADC           */
  BEGIN                                /* ---------------------------------- */
    UCHAR         adc_num            ; /* Timer number for accurate channel  */
    UCHAR         tim_num            ; /* Timer number for accurate channel  */
    UCHAR         dma_num            ; /* DMA reference number               */
    UCHAR         jext_trig          ; /* Extern trigger for injected channel*/
    UCHAR         accu_ch            ; /* Number of the accurate channel     */
    UINT          dma_ch             ; /* DMA reference channel              */
    UCHAR         dma_st             ; /* DMA reference stream               */
  END             Adc_stm32_def      ; /* ---------------------------------- */

typedef struct                         /* Low level Context                  */
  BEGIN                                /* ---------------------------------- */
    UCHAR         dev                ; /* Device index for ADC driver        */
    UCHAR         adc_num            ; /* ADC number                         */
    UCHAR         tim_num            ; /* TIM number                         */
    UCHAR         dma_num            ; /* DMA number                         */
    UCHAR         jext_trigg         ; /* external trig for injected channel */
    UCHAR         accu_ch            ; /* accurate channel number            */
    UINT          width              ; /* Width                              */
    UINT          wmask              ; /* mask associated to the width       */
    volatile Tim *timaddr            ; /* Timer address                      */
    UINT          freq               ; /* Timer frequency                    */
  END             Adc_stm32          ; /* ---------------------------------- */


/*
    3    Public variables
    _____________________
*/


extern const Adc_desc adc_desc_stm32 ; /* ADC  low level interface           */
extern       Adc_stm32 adc_stm32[]   ; /* Context table of tim_stm32.c       */


/*
    4    Prototypes of public procedures
    ____________________________________
*/


/* Declaration of the procedures ........................................... */

IWORD handlerADC_stm32                 (IWORD,IWORD                          );
IWORD handlerDAC_stm32                 (IWORD,IWORD                          );
IWORD handlerdmaDAC_stm32              (IWORD,IWORD                          );


#endif                                 /* NEW_ADC                            */

#endif                                 /* End of ADC_STM32_H                 */
