/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS                       File "include/drv_int.h"             */
/*  Constants and definitions for IT driver.                                 */
/*                                                                           */
/*  Author : JYA                        Last modification : 14/11/2017       */
/*                                                                           */
/*  (c) Copyright Hyperpanel Lab, All Rights Reserved                        */
/*  IDDN.FR.001.320001.*.S.P.1990.000.10000                                  */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations ========================================================


         1  Defines
         2  Structures and typedefs
            2.1  Definition of "ithead" and "ithand" tables
            2.2  Configuration structures
         3  Public variables
         4  Prototypes of public procedures
            4.1  Procedures provided by "drv_int.c"
            4.2  Low level procedure required by "drv_int.c"


*/


#ifndef DRV_INT_H                      /* If this file is not already        */
#define DRV_INT_H                      /* included, include it.              */

 
/*
    1    Defines
    ____________
*/


#define            MAX_ITHAND 64       /* Taille maximale de "ithand/itdec"  */


/*
    2    Structures and typedefs
    ____________________________
*/


/*
    2.1  Definition of "ithead" and "ithand" tables
    -----------------------------------------------
*/


typedef struct                         /* Une entree de "ithead[]"           */
  BEGIN                                /* ---------------------------------- */
    UINT            mask1            ; /* Le bit correspondant au fil d'IT   */
    UINT            mask2            ; /* Suite du mot precedent             */
    IWORD           hand             ; /* Indice dans la table "ithand"      */
  END               Ithead_t         ; /* ---------------------------------- */

typedef struct                         /* Une entree de "ithand[]"           */
  BEGIN                                /* ---------------------------------- */
    UINT            next             ; /* Numero prochain handler meme IT    */
    IWORD           (*proc)()        ; /* Adresse procedure traitement IT    */
    IWORD           nctx             ; /* Contexte                           */
    IWORD           ntype            ; /* Type                               */
  END               Ithand_t         ; /* ---------------------------------- */


/*
    2.2  Configuration structures
    -----------------------------
*/


typedef struct                         /* Procedures de bas niveau DRV_INT   */
  BEGIN                                /* ---------------------------------- */
    void            (*init_vector)() ; /* Adresse procedure "init_vector_xyz"*/
    void            (*end_vector )() ; /* Adresse procedure "end_vector_xyz" */
    IWORD           (*intctl     )() ; /* Adresse procedure "intctl_xyz"     */
    void            (*softit     )() ; /* Adresse procedure "softit_xyz"     */
  END               Int_desc         ; /* ---------------------------------- */

typedef struct                         /* Regroupement des structures        */
  BEGIN                                /* ---------------------------------- */
    const IWORD    *idx              ; /* Index des niveaux dans ithead      */
    const Ithead_t *head             ; /* Adresse table 1er traitement IT    */
    const Ithand_t *hand             ; /* Adresse table infos handlers       */
  END               Itdev_t          ; /* ---------------------------------- */

typedef struct                         /* Donnees liees a une IT             */
  BEGIN                                /* ---------------------------------- */
    IWORD          *adstack          ; /* Adresse de la pile a utiliser      */
    IWORD           szstack          ; /* Taille en octets de la pile        */
    IWORD           flag             ; /* Bits 0 du HPTR                     */
    IWORD           trigger          ; /* Trigger mode                       */
  END               Itdata_t         ; /* ---------------------------------- */


/*
    3  Public variable
    __________________
*/


extern const Sdrv_desc  sdrv_int   ; /* System driver description          */
extern const Int_desc   int_desc_nvic  ;

extern IWORD        itnext           ; /* Numero handler suivant.            */
extern IWORD        itdec[MAX_ITHAND]; /* Decompteurs d'ITs                  */
extern volatile     UCHAR itcurrprio ; /* Priorite de l'IT en cours.         */



/*
    4    Prototypes of public procedures
    ____________________________________
*/


/* 4.1  Procedures provided by "drv_int.c"...................................*/

  IWORD init_int            (IWORD                                           );
  IWORD end_int             (IWORD                                           );
  void  cliIT               (void                                            );
  void  stiIT               (void                                            );
  IWORD chkIT               (void                                            );
  void  softIT              (IWORD                                           );
  IWORD getindIT            (IWORD (*)(),IWORD                               );
  IWORD int_ctl             (UINT,IWORD                                      );
  void  int_proc            (UINT                                            );


/* 4.2  Low level procedure required by "drv_int.c" .........................*/

  void  enable_gint         (void                                            );
  void  disable_gint        (void                                            );
  void  set_vbr             (void(*)()                                       );
  void  exception           (void                                            );


#endif                                 /* End of DRV_INT_H.                  */
