/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS                       File "include/file.h"                */
/*  Constants and definitions for file system.                               */
/*                                                                           */
/*  Author : JYA                        Last modification : 04/11/2016       */
/*                                                                           */
/*  (c) Copyright Hyperpanel Lab, All Rights Reserved                        */
/*  IDDN.FR.001.320001.*.S.P.1990.000.10000                                  */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations ========================================================


         1  Defines
         2  Structures and typedefs
         3  Public variables
         4  Prototypes of public procedures


*/


#ifndef FILE_H                         /* If this file is not already        */
#define FILE_H                         /* included, include it.              */


/*
    1    Defines
    ____________
*/


/* General ..................................................................*/

#define HYP_INI       "hyp.ini"        /* Fichier de config. de "run".       */
#define AM_INI         "am.ini"        /* Fichier de config. de "run".       */
#define VHOME            "HOME"        /* Volume systeme, identificateur.    */
#define VLOCAL          "LOCAL"        /* Volume systeme, identificateur.    */
#define VEXE              "EXE"        /* Volume systeme, identificateur.    */

#define PHOME          (INT)100        /* Volume systeme, priorite.          */
#define PLOCAL         (INT)101        /* Volume systeme, priorite.          */
#define PEXE           (INT)102        /* Volume systeme, priorite.          */
#define DOWNLOADED_MOD (INT)0x40       /* Flag module telecharge             */
#define MD5RSA        (INT)0x80        /* MD5 du repertoire crytpe oui/non   */
#define BLOC_HDR              6        /* Taille en-tete bloc fichier RAM    */
#define NB_MPEG2_TAGS         9        /*                                    */
#define DOS_BUFREAD        0x10        /* DOS Nb de blocs bufferises         */


/* Mnemoniques d'etat d'un traitement .......................................*/

#define DF_DONE               0        /* Traitement termine                 */
#define DF_ASYNC              1        /* Traitement suspendu                */


/* Description d'un fichier .................................................*/

#define VOL_FILSZETOT   (INT)20        /* Taille description d'1 fichier     */
#define VOL_FILSZENOM   (INT)12        /* Taille du nom du fichier           */
#define VOL_FILSZEPOS    (INT)4        /* Taille de la position du fichier   */
#define VOL_FILSZESZE    (INT)4        /* Taille de la taille du fichier     */

#define VOL_FILOFFNOM    (INT)0        /* Offset du nom du fichier           */
#define VOL_FILOFFPOS   (INT)12        /* Offset de la position du fichier   */
#define VOL_FILOFFSZE   (INT)16        /* Offset de la taille du fichier     */


/* Donnees telechargees 1  : MD5 par module, MD5/RSA global .................*/

                                       /* Description d'un repertoire        */
#define VOL1_DIRSZETOT  (INT)30        /* Taille totale d'un directory       */
#define VOL1_DIRSZENOM   (INT)8        /* Taille du nom dans le module       */
#define VOL1_DIRSZERES   (INT)2        /* Taille du champ reserve            */
#define VOL1_DIRSZEVER   (INT)1        /* Taille du numero de version        */
#define VOL1_DIRSZEIND   (INT)2        /* Taille de l'indice du module       */
#define VOL1_DIRSZESIG  (INT)16        /* Taille de la signature non cryptee */
#define VOL1_DIRSZEATT   (INT)1        /* Taille des attributs               */

#define VOL1_DIROFFNOM   (INT)0        /* Offset du nom dans le module       */
#define VOL1_DIROFFRES   (INT)8        /* Offset du champ reserve            */
#define VOL1_DIROFFVER  (INT)10        /* Offset du numero de version        */
#define VOL1_DIROFFIND  (INT)11        /* Offset de l'indice du module       */
#define VOL1_DIROFFSIG  (INT)13        /* Offset de la signature non cryptee */
#define VOL1_DIROFFATT  (INT)29        /* Offset des attributs               */


/* Description donnees telechargees 2  : MD5/RSA par module .................*/

                                       /* Description d'un repertoire        */
#define VOL2_DIRSZETOT  (INT)79        /* Taille totale d'un directory       */
#define VOL2_DIRSZENOM   (INT)8        /* Taille du nom dans le module       */
#define VOL2_DIRSZERES   (INT)2        /* Taille du champ reserve            */
#define VOL2_DIRSZEVER   (INT)1        /* Taille du numero de version        */
#define VOL2_DIRSZEIND   (INT)2        /* Taille de l'indice du module       */
#define VOL2_DIRSZESIG  (INT)64        /* Taille de la signature cryptee     */
#define VOL2_DIRSZECLE   (INT)1        /* Taille du numero de cle            */
#define VOL2_DIRSZEATT   (INT)1        /* Taille des attributs               */

#define VOL2_DIROFFNOM   (INT)0        /* Offset du nom dans le module       */
#define VOL2_DIROFFRES   (INT)8        /* Offset du champ reserve            */
#define VOL2_DIROFFVER  (INT)10        /* Offset du numero de version        */
#define VOL2_DIROFFIND  (INT)11        /* Offset de l'indice du module       */
#define VOL2_DIROFFSIG  (INT)13        /* Offset de la signature cryptee     */
#define VOL2_DIROFFCLE  (INT)77        /* Offset du numero de cle            */
#define VOL2_DIROFFATT  (INT)78        /* Offset des attributs               */


/* Numeros des disques RAM et ROM ...........................................*/

#define  MDF_DISKRAM          0        /* Disque RAM                         */
#define  MDF_DISKROM          1        /* Disque ROM                         */


/* MHEG - DSMCC .............................................................*/

#define NBDII                 5        /*                                    */
#define SIZEOF_HMPEG2         8        /*                                    */
#define SIZEOF_HDSMCC        12        /*                                    */
#define MS                   16        /*                                    */


/* Redefinitions ............................................................*/

#define Mfdir    Df_fdir               /*                                    */
#define Df_from  Df_fdir               /*                                    */


/*
    2    Structures and typedefs
    ____________________________
*/


/* Table des systemes de fichiers ...........................................*/

typedef struct                         /* Interface DF -> driver de volume   */
  BEGIN                                /* ---------------------------------- */
    CHAR            name[STDNAME]    ; /* Nom du driver de volume            */
    CHAR            typdrv           ; /* 6 : code d'un driver de volume     */
    IWORD           (*drv_evt  ) ()  ; /* Ad procedure remontee evenement    */
    IWORD           (*init     ) ()  ; /* Initialisation du driver de volume */
    IWORD           (*mount    ) ()  ; /* Monter une partition disque        */
    IWORD           (*umount   ) ()  ; /* Demonter un partition disque       */
    IWORD           (*volinfo  ) ()  ; /* Lire informations d'un volume monte*/
    IWORD           (*credir   ) ()  ; /* Creer un repertoire                */
    IWORD           (*deldir   ) ()  ; /* Detruire un repertoire             */
    IWORD           (*crefile  ) ()  ; /* Creer un fichier                   */
    IWORD           (*delfile  ) ()  ; /* Detruire un fichier                */
    IWORD           (*renfile  ) ()  ; /* Renommer un fichier                */
    IWORD           (*filelist ) ()  ; /* Liste des fichiers d'un repertoire */
    IWORD           (*fileinfo ) ()  ; /* Informations d'un fichier          */
    IWORD           (*open     ) ()  ; /* Ouvrir un fichier                  */
    IWORD           (*close    ) ()  ; /* Fermer un fichier                  */
    IWORD           (*pseek    ) ()  ; /* Seek sur fichier ouvert en ecriture*/
    IWORD           (*put      ) ()  ; /* Ecrire un fichier                  */
    IWORD           (*gseek    ) ()  ; /* Seek sur fichier ouvert en lecture */
    IWORD           (*get      ) ()  ; /* Lire un fichier                    */
    IWORD           (*ioctl    ) ()  ; /* Operations de controles            */
    UCHAR           tvol             ; /* Type du driver de volume RAM/ROM ..*/
    UCHAR           async            ; /* 0/1 synchrone/asynchrone           */
  END               Df_driver        ; /* ---------------------------------- */


/* Table des disques ........................................................*/

typedef struct                         /* Disque disponible, initialise      */
  BEGIN                                /* ---------------------------------- */
    CHAR            name[MAX_DFNAME+1];/* Nom du disque                      */
    UCHAR           major            ; /* Numero du module file system       */
    INT             dskno            ; /* Identifiant pour le file system    */
    INT             iodb             ; /* IOD device bloc                    */
    UCHAR           ro               ; /* 0: RW, 1: RO                       */
  END               Df_dsk           ; /* ---------------------------------- */


/* Repertoire de fichiers ...................................................*/


/*****************************************************************************
 * Pour chaque module du volume ROM, on a une liste de fichiers, qui est un  *
 * tableau de structures Df_fdir. Attention : ne pas ajouter de +1 a la      *
 * taille de "name" Dans le cas ou le nom a une taille de 12 caracteres, il  *
 * n'y a pas de 0 a la fin.                                                  *
 *****************************************************************************/


typedef struct                         /* Definition entree directory fichier*/
  BEGIN                                /*                                    */
    CHAR      name[MAX_DFNAME+STDEXT]; /* Nom du fichier                     */
    UINT      pos                    ; /* position du debut du fichier       */
    UINT      size                   ; /* taille du fichier                  */
  END         Df_fdir                ; /*                                    */


/*****************************************************************************
 * Pour chaque module du volume RAM, on a une liste de fichiers, qui est un  * 
 * tableau de structures Df_fram.                                            *
 *****************************************************************************/


typedef struct                         /* Definition entree directory fichier*/
  BEGIN                                /*                                    */
    CHAR    name[ MAX_DISKPATH + 1 ] ; /* Nom du fichier                     */
    UINT    pos                      ; /* Position du debut du fichier       */
    UINT    size                     ; /* Taille du fichier                  */
  END      Df_fram                   ; /*                                    */


/* Repertoire de modules ....................................................*/


/*****************************************************************************
 * Chaque repertoire de module est un tableau de structures Df_mdir.         *
 * Attention : ne pas ajouter de +1 a la taille de "name" Dans le cas ou le  *
 * nom a une taille de 8 caracteres, il n'y a pas de 0 a la fin.             *
 *****************************************************************************/


typedef struct                         /* Definition entree directory modules*/
  BEGIN                                /*                                    */
    CHAR        name[MAX_DFNAME]     ; /* Nom du repertoire                  */
    UCHAR       *data                ; /* Adr donnees 1er fichier ou HNULL   */
    UCHAR       version              ; /* Numero de version                  */
    UCHAR       att                  ; /* Attributs du module                */
    UCHAR       nkey                 ; /* Numero clef RSA de telechargement  */
    UCHAR       device               ; /* Numero device de telechargement    */
    UCHAR       sig[VOL2_DIRSZESIG]  ; /* Signature MD5 du module cryptee ou */
  END           Df_mdir              ; /* non.                               */


/* Table des volumes montes .................................................*/

typedef struct
  BEGIN                                /*                                    */
    CHAR        name[MAX_DFNAME+1]   ; /* Nom donne au volume 8 cara max     */
    CHAR        mpath[MAX_DISKPATH+1]; /* Path de montage du volume          */
    Df_mdir    *mdir                 ; /* Adresse liste des modules          */
    USHORT      nbm                  ; /* Nombre de modules                  */
    UCHAR      *path                 ; /* Ad Pathname ou HNULL               */
    UCHAR       major                ; /* Major number du drvier de volume   */
    UCHAR       tvol                 ; /* Type du volume RAM-ROM/HDD-FLOPPY. */
    UCHAR       diskid               ; /* Numero dans la table des disques   */
    UCHAR       ttext                ; /* Type des fichiers textes           */
    UCHAR       priority             ; /* Priorite                           */
    UCHAR       ro                   ; /* READ ONLY : 0/1                    */
  END           Df_vol               ; /*                                    */
 

/* Contexte d'interpretation ................................................*/


/*****************************************************************************
 * Description volume, repertoire ou fichier que l'on veut consulter ou      *
 * modifier                                                                  *
 *****************************************************************************/


typedef struct                         /* Volume, repertoire ou fichier.     */
  BEGIN                                /* ---------------------------------- */
    UCHAR           vid              ; /* Identifiant de volume              */
    IWORD           fid              ; /* Identifiant de fichier ouvert      */
    USHORT          did              ; /* Identifiant de repertoire          */
    CHAR            name[MAX_DISKPATH+1] ; /* Nom volume/repertoire/fichier  */
    INT             iod              ; /* IOD de fichier                     */
    UCHAR           mode             ; /* Flags "texte" du volume            */
    UCHAR           over             ; /* Flag "overwrite" du volume         */
    UCHAR           tlc              ; /* Flag buffer telecom                */
    UCHAR          *adr              ; /* Adresse buffer associe             */
    UINT            first            ; /* Offset premier octet de donnee     */
    UINT            sz               ; /* Taille du buffer                   */
    UINT            nd               ; /* Nombre d'octets de donnees         */
    UINT            lgnxt            ; /* Taille max prochaine copie         */
  END               Vdf_ctx          ; /* ---------------------------------- */

typedef struct                         /* Description d'un fichier.          */
  BEGIN                                /* ---------------------------------- */

    /* Contexte d'execution lors de l'interruption ..........................*/

    S_evt           evt              ; /* Evenement de reponse               */
    IWORD           fifo             ; /* FIFO depot evt de reponse          */
    IWORD           (*cb)()          ; /* Proc MAJ parametres de retour      */
    INT             cbpar            ; /* Parametre procedure "cb"           */
    UCHAR           suspended        ; /* Flag d'execution interrompue       */
    UCHAR           nbl              ; /* Flag appel PANTALK non bloquant    */
    INT            *aret             ; /* Adresse variable "ret"             */
    INT            *pret1            ; /* Adresses 1er et 2eme parametre     */
    INT            *pret2            ; /* de retour.                         */

    /* Descriptions des volumes, repertoires ou fichiers ....................*/

    Vdf_ctx         dsc[2]           ; /* Descriptions de fichiers           */
    Vdf_ctx        *cd               ; /* Description de fichier courante    */
    Df_vol         *pv               ; /* Description du volume courant      */
    Df_driver      *dv               ; /* Interface driver de volume         */
    INT             dskno            ; /* Id driver de volume partition      */

    /* Donnees utilisees par l'interpreteur lui-meme ........................*/

    UINT            iptr             ; /* Pointeur d'instruction             */
    UINT            pred             ; /* Valeur du predicat                 */
    INT             cond             ; /* Derniere condition positionnee     */
    INT             par              ; /* Parametre a semantique non         */
    INT             err              ; /* defini. Code d'erreur.             */
    INT             ncpy             ; /* Nombre d'octets a copier           */
    USHORT          ndf              ; /* Nombre de repertoires/fichiers     */
    UINT            seek             ; /* Position de saut                   */
    UINT            newpos           ; /* Nouvelle position                  */
    UCHAR           mode             ; /* Mode retour contenu repertoire     */
    UCHAR           eol              ; /* Flag de fin de ligne               */
    UCHAR           pmin             ; /* Priorite minimale et maximale de   */
    UCHAR           pmax             ; /* recherche sur les volumes.         */
    CHAR            ext[STDEXT+1]    ; /* Extension de fichier               */
  END               Df_ctx           ; /* ---------------------------------- */


/* Table des fichiers ouverts ...............................................*/

typedef struct                         /* Bloc courant (compresse ou decomp) */
  BEGIN                                /* ---------------------------------- */
    UCHAR          *buf              ; /* Adresse du buffer contenant le bloc*/
    UCHAR          *deb              ; /* Adresse de debut du bloc           */
    UCHAR          *cour             ; /* Adresse courante dans le bloc      */
    UCHAR          *fin              ; /* Adresse de fin du bloc             */
    INT             pos              ; /* Position dans le fichier           */
    INT             fsize            ; /* Taille du fichier                  */
    IWORD           tlc              ; /* Flag buffer telecom a liberer      */
  END               Df_bloc          ; /* ---------------------------------- */

typedef struct                         /* Description d'un fichier ouvert    */
  BEGIN                                /* ---------------------------------- */
    INT             iod              ; /* IOD fichier ouvert                 */
    IWORD           fd               ; /* File descriptor                    */
    USHORT          mode             ; /* Modes d'acces                      */
    UCHAR           volid            ; /* Indice du volume                   */
    USHORT          modid            ; /* Indice du dir. dans le volume      */
    INT             inbio            ; /* Contexte de lecture de bits        */
    INT             outbio           ; /* Contexte d'ecriture de bits        */
                 
    Df_bloc         b[2]             ; /* 0: description du bloc courant     */
                                       /*    compresse ou non                */
                                       /* 1: dans le cas d'un fichier        */
                                       /*    compresse, bloc decompresse     */
                                       /*    courant.                        */

    INT             fpos             ; /* Position debut fichier/ deb module */
    UCHAR           mdisk            ; /* Id du disque memoire               */
    UCHAR          *mptr             ; /* Adresse debut du module            */
    UCHAR           pid              ; /* Numero protocole de telechargement */
  END               Df_o             ; /* ---------------------------------- */

typedef struct                         /* Definition DISK en ROM             */
  BEGIN                                /* ---------------------------------- */
    CHAR            *disk            ; /* Nom de la partition                */
    CHAR            *fsystem         ; /* Nom du module systeme de fichier   */
    CHAR            *driver          ; /* Nom du driver bloc                 */
    INT             dev              ; /* Le device                          */
    INT             part             ; /* Numero de la partition             */
  END               ROM_disk         ; /* ---------------------------------- */
 
typedef struct                         /* Definition VOLUME en ROM           */
  BEGIN                                /* ---------------------------------- */

    CHAR           *name             ; /* Nom du volume                      */
    CHAR           *disk             ; /* Nom de la partition                */
    CHAR           *pere             ; /* Nom du volume pere                 */
    CHAR           *path             ; /* Nom du chemin de montage           */
    UCHAR           flags            ; /* Type des fichiers texte            */
    INT             pri              ; /* Priorite                           */
  END               ROM_vol          ; /* ---------------------------------- */

typedef struct                         /* Description contexte automate      */
  BEGIN                                /* ---------------------------------- */
    IWORD       aut                  ; /* Numero automate appelant           */
    IWORD       vl                   ; /* Numero VL automate appelant        */
    SHORT       code                 ; /* Champ code evenement de fin        */
    INT         res                  ; /* Champ reserve evenement de fin     */
    INT         *err                 ; /* adresse parametre code d'erreur    */
    SHORT       modid                ; /* identificateur du module           */
    UCHAR       volid                ; /* identificateur du volume           */
    INT         reqid                ; /* numero de requete                  */
    INT         iod                  ; /* IOD utilise pour la requete        */
    CHAR        name[MAX_DFNAME+1]   ; /* nom du volume/module               */
  END           Dnl_ctx              ; /* ---------------------------------- */

typedef struct                         /* Description librairie              */
 BEGIN                                 /* ---------------------------------- */
  CHAR      file[MAX_DFNAME+STDEXT+1]; /* Nom           du fichier librairie */
  UCHAR     volid                    ; /* Id volume     du fichier librairie */
  USHORT    dirid                    ; /* Id repertoire du fichier librairie */
  UCHAR     vidf                     ; /* Id volume DF correspondant         */
  Df_fdir **fls                      ; /* Table des listes de fichiers       */
  SHORT    *fnb                      ; /* Table des nombre de fichiers       */
 END        Df_lib                   ; /* ---------------------------------- */

typedef struct                         /* Description contexte MPEG          */
  BEGIN                                /* ---------------------------------- */
    INT             vid              ; /* Volume DF concerne                 */
    INT             dnl              ; /* Contexte DNL global                */
    INT             iod              ; /* IOD canal telecom                  */
    INT             bloc             ; /* Numero du bloc charge              */
    INT             to               ; /* Identifiant de la temporisation    */
    INT             lgbuf            ; /* Longueur des buffers a allouer     */
    INT             nbbuf            ; /* Nombre   de  buffers alloues       */
    UCHAR          *adr[256]         ; /* Adresses des buffers alloues       */
    UCHAR           rcv[256]         ; /* Index des sections dans les buffers*/
  END               Dnl_mpg          ; /* ---------------------------------- */


/* Structures MHEG - DSMCC ..................................................*/

typedef struct             /* Description d'un flot d'entree/sortie          */
 BEGIN                     /* ---------------------------------------------- */
  IWORD          mode    ; /* 0 buffer, 1 fichier, 2 MDF                     */
  UCHAR         *ptr     ; /* Pointeur sur l'octet suivant.                  */
  UCHAR         *end     ; /* Pointeur sur l'octet inutilisable.             */
  IWORD          df      ; /* Identifiant de fichier (-1 si inutilise).      */
  UCHAR          byte    ; /* Valeur de l'octet courant.                     */
  IWORD          bits    ; /* Compte les bits consommes de l'octet courant.  */
  UCHAR*         deb     ; /* Adresse de debut   du bloc courant             */
  UCHAR*         cour    ; /* Adresse de lecture du bloc courant             */
  UCHAR*         bend    ; /* Adresse de fin     du bloc courant             */
 END Dio                 ; /* ---------------------------------------------- */


typedef struct             /* Definition d'un header de section MPEG2        */
 BEGIN                     /* ---------------------------------------------- */
  UCHAR          tid     ; /* Table Id            (0xFF interdit)            */
  USHORT         lg      ; /* Longueur totale - 3 (maximum 0xFFF)            */
  USHORT         tid_ext ; /* Table Id extension                             */
  UCHAR          ver     ; /* Version de la section                          */
  UCHAR          sec     ; /* Numero  de la section                          */
  UCHAR          lsec    ; /* Numero  de la derniere section                 */
 END Hmpeg2              ; /* ---------------------------------------------- */


typedef struct             /* Definition d'un header de message DSMCC        */
 BEGIN                     /* ---------------------------------------------- */
  UCHAR          proto   ; /* ProtocolDiscriminator                11        */
  UCHAR          type    ; /* Type                                 03        */
  USHORT         msgid   ; /* Identifiant du type de message                 */
  UIWORD         traid   ; /* Identifiant de la transaction                  */
  UCHAR          adapt   ; /* Longueur du champ d'adaptation                 */
  USHORT         lg      ; /* Longueur du message                            */
 END Hdsmcc              ; /* ---------------------------------------------- */


typedef struct             /* Definition d'une reference d'objet DSMCC       */
 BEGIN                     /* ---------------------------------------------- */
  UIWORD         carid   ; /* carouselId                                     */
  UIWORD         traid   ; /* transactionId                                  */
  UIWORD         modsz   ; /* moduleSize                                     */
  UIWORD         modos   ; /* original size (== modsz si pas compresse)      */
  USHORT         modid   ; /* moduleId                                       */
  USHORT         blksz   ; /* blockSize                                      */
  IWORD          tag     ; /* TAG                                            */
  UCHAR          modve   ; /* moduleVersion                                  */
  UCHAR          nam[65] ; /* Nom                                            */
  UCHAR          key[20] ; /* Cle                                            */
  IWORD          diiid   ; /* Identifiant du DII (PID dans le cas MPEG2)     */
 END Dsmcc_ref           ; /* ---------------------------------------------- */


typedef struct             /* Definition d'un protocole de bas niveau.       */
 BEGIN                     /* ---------------------------------------------- */
  CHAR   name[9]         ; /*                                                */
  IWORD (*init      )()  ; /*                                                */
  IWORD (*mount     )()  ; /*                                                */
  IWORD (*umount    )()  ; /*                                                */
  IWORD (*fil_load  )()  ; /*                                                */
  IWORD (*fil_unload)()  ; /*                                                */
  UCHAR *tag             ; /*                                                */
 END Dsmcc_pro           ; /* ---------------------------------------------- */


typedef struct             /* Definition d'un Carousel.                      */
 BEGIN                     /* ---------------------------------------------- */
  UCHAR   etat           ; /* Etat du carousel.                              */
  UCHAR   dir[9]         ; /* Nom du repertoire dans le disque RAM.          */
  INT     pid            ; /* Identifiant du protocole de bas niveau.        */
  Dsmcc_pro *pro         ; /* Interface   du protocole de bas niveau.        */
  INT     res            ; /* Champ reserve de l'evenement de fin.           */
  SHORT   mod            ; /* Identifiant du module du gateway.              */
  USHORT  str_idt[257]   ; /* Identifiant des streams taggees.               */
  INT     nfiles         ; /* Nombre de fichiers en cours de chargement.     */
  INT     modid          ; /* Identifiant du module du gateway.              */
  UCHAR   key[20]        ; /* Cle du SRG                                     */
  UINT reg[NB_MPEG2_TAGS]; /* Registres                                      */
 END Dsmcc_car           ; /* ---------------------------------------------- */


typedef struct             /* Definition d'un Fichier.                       */
 BEGIN                     /* ---------------------------------------------- */
  UCHAR   car            ; /* Identifiant du carousel.                       */
  UCHAR   fil[67]        ; /* Nom du fichier a charger                       */
  INT     ofd            ; /* Offset de debut de recherche dans le nom       */
  INT     off            ; /* Offset de fin   de recherche dans le nom       */
  INT     res            ; /* Champ reserve  de l'evenement de fin           */
  INT     lg             ; /* Champ longueur de l'evenement de fin           */
  IWORD   reload         ; /* Flag de rechargement                           */
 END Dsmcc_fil           ; /* ---------------------------------------------- */


typedef struct             /* Definition d'une reference d'objet DSMCC       */
 BEGIN                     /* ---------------------------------------------- */
  UIWORD         carid   ; /* carouselId                                     */
  UIWORD         traid   ; /* transactionId                                  */
  UIWORD         modsz   ; /* moduleSize                                     */
  UIWORD         modos   ; /* original size (== modsz si pas compresse)      */
  USHORT         modid   ; /* moduleId                                       */
  USHORT         blksz   ; /* blockSize                                      */
  IWORD          tag     ; /* TAG                                            */
  UCHAR          modve   ; /* moduleVersion                                  */
  UCHAR          nam[65] ; /* Nom                                            */
  UCHAR          key[20] ; /* Cle                                            */
 END Rdsmcc              ; /* ---------------------------------------------- */


typedef struct             /* Definition d'un objet service gateway          */
 BEGIN                     /* ---------------------------------------------- */
  INT     dnl            ; /* Contexte de chargement                         */
  INT     sid            ; /* Service ID associe                             */
  INT     res            ; /* Champ reserve de l'evenement de fin            */
  INT     iob            ; /* IOD canal telecom                              */
  USHORT  pids[257]      ; /* PID/tag                                        */
  USHORT  dii_pid[NBDII] ; /* PID    du DII                                  */
  USHORT  dii_tex[NBDII] ; /* TIDEXT du DII                                  */
  UCHAR  *dii_adr[NBDII] ; /* Buffer du DII                                  */
  UCHAR   dii_ers        ; /* Id du prochain DII a effacer                   */
  UCHAR   dir[9]         ; /* Nom du repertoire dans le volume RAM           */
 END Mheg_srg            ; /* ---------------------------------------------- */


typedef struct             /* Definition d'une requete de chargement         */
 BEGIN                     /* ---------------------------------------------- */
  INT          dnl       ; /* Contexte de chargement                         */
  INT          srg       ; /* Voie logique du ServiceGateway                 */
  INT          res       ; /* Champ reserve de l'evenement de fin            */
  INT          ofd       ; /* Offset de debut de recherche dans le nom       */
  INT          off       ; /* Offset de fin   de recherche dans le nom       */
  USHORT       lg        ; /* Champ longueur de l'evenement de fin           */
  UCHAR        fil[65]   ; /* Nom du fichier a charger                       */
 END Mheg_fil            ; /* ---------------------------------------------- */


typedef struct             /* Definition d'une voie logique de chargement    */
 BEGIN                     /* ---------------------------------------------- */
  USHORT       pid       ; /* PID     en cours de chargement                 */
  USHORT       tid       ; /* TID     en cours de chargement                 */
  USHORT       tex       ; /* TID_EXT en cours de chargement                 */
  INT          iod       ; /* IOD     canal telecom                          */
  INT          to        ; /* Voie logique de chargement associee            */
  IWORD        lgbuf     ; /* Longueur des buffers alloues                   */
  UCHAR        nbbuf     ; /* Nombre de buffers alloues                      */
  UCHAR       *adr[MS]   ; /* Adresses des buffers alloues                   */
  CHAR         rcv[MS]   ; /* Numero   des buffers par section               */
  Rdsmcc       rdsmcc    ; /* Reference a un objet Dsmcc                     */
 END Mheg_dnl            ; /* ---------------------------------------------- */


/* Constantes et structures DOS .............................................*/

typedef struct                         /* Partition de device physique.      */
 BEGIN                                 /* ---------------------------------- */
    IWORD           id               ; /* Id du disque                       */
    USHORT          canal            ; /* Numero canal physique              */
    UCHAR           driv             ; /* Numero du driver                   */
    IWORD           devp             ; /*                                    */
    INT             deb              ; /* Numero premier bloc                */
    INT             fin              ; /* Numero dernier bloc                */
    INT             size             ; /* Taille totale en blocs             */
                                       /* Device logique                     */
    IWORD           nbsc             ; /* Nb de secteurs par clusters        */
    IWORD           nbos             ; /* Nb d'octets par secteurs           */
    UCHAR           fat              ; /* Type de fat                        */
    UCHAR           nbfat            ; /* Nombre de FATs                     */
    INT             sfat1            ; /* Numero relatif 1er bloc FAT1       */
    INT             sfat2            ; /* Numero relatif 1er bloc FAT2       */
    INT             sifat            ; /* Taille d'une FAT                   */
    INT             sres             ; /* Nombre de blocs reserves           */
    INT             srep             ; /* Numero relatif 1er bloc rep.       */
    INT             sdon             ; /* Numero relatif 1er bloc don.       */
    INT             sirep            ; /* Nb d'entrees repertoire root       */
    INT             avl              ; /* Flag d'available pour floppys      */
    UCHAR          *buf              ; /* Buffer pour FAT...                 */
    INT             nsc              ; /* Numero secteur dans buffer         */
    CHAR            ok               ; /* Flag d'occupe                      */
    CHAR            name[MAX_DFNAME] ; /* Nom du disque                      */
    UCHAR          *read_buf         ; /* Buffer lecture                     */
    INT             read_s[DOS_BUFREAD]; /* Secteur du buffer.               */
    UINT            read_a[DOS_BUFREAD]; /* Age de la lecture.               */
    UINT            read_age         ; /* Age courant en lecture.            */
 END Dos_devl                        ; /* ---------------------------------- */

typedef struct                         /* Definition d'un fichier ouvert     */
  BEGIN                                /* ---------------------------------- */
    UCHAR           vol              ; /* Numero du device logique           */
    INT             flag             ; /* Mode d'ouverture                   */
    INT             s                ; /* Numero du premier secteur          */
    INT             size             ; /* Taille en octets                   */
    INT             pos              ; /* Position buffer dans fichier       */
    UCHAR          *buf              ; /* Buffer d'anticipation              */
  END               Dos_openf        ; /* ---------------------------------- */

typedef struct                         /* Definition du volume               */
  BEGIN                                /* ---------------------------------- */
    UCHAR           vol              ; /* Id df du volume                    */
    UCHAR           dev              ; /* Numero du device logique           */
    INT             sec              ; /* Numero premier secteur             */
    INT             size             ; /* Nbre d'entrees                     */
    CHAR           *p                ; /* Path du repertoire                 */
  END               Dos_vol          ; /* ---------------------------------- */

typedef struct                         /* Description d'un fichier en FLASH  */
  BEGIN                                /* ---------------------------------- */
    UCHAR           ignore           ; /* Ignore cette description           */
    UCHAR           etat             ; /* Etat du fichier                    */
    UCHAR           version          ; /* Version du fichier                 */
    CHAR            name[MAX_DFNAME+STDEXT+1] ; /* Nom  du fichier           */
    UINT            offset           ; /* Offset du fichier dans le bloc en  */
    UINT            alloc            ; /* octets. Taille du fichier en octets*/
  END               Fdf_file         ; /* ---------------------------------- */

typedef struct                         /* Disque FLASH.                      */
  BEGIN                                /* ---------------------------------- */

    struct file                        /* Description fichier en cours de    */
      BEGIN                            /* traitement pour creation, efface,  */
        UCHAR       ignore           ; /* ouverture ou fermeture.            */
        UCHAR       etat             ; /*                                    */
        UCHAR       version          ; /*                                    */
        CHAR        name[13]         ; /*                                    */
        UINT        offset           ; /*                                    */
        UINT        alloc            ; /*                                    */
        INT         iod              ; /* IOD du fichier ouvert              */
        UINT        pos              ; /* Position de lecture / ecriture     */
        UCHAR     *shd               ; /* Cache des donnees d'un fichier     */
      END file                       ; /*                                    */
  
    S_evt           req              ; /*                                    */
    UCHAR           dskid            ; /* Identifiant client disque FLASH    */
    USHORT          reqemet          ; /* Num automate emetteur de requete   */
    USHORT          reqevl           ; /* VL emettrice de la requete         */
    USHORT          reqid            ; /* Identifiant utilisateur de requete */
    UCHAR          *abl              ; /* Adresse du 1er bloc.               */
    IWORD           nbl              ; /* Nombre total de blocs.             */
    IWORD           sbl              ; /* Taille d'un bloc.                  */
    IWORD           mfl              ; /* Nombre max de fichiers par bloc.   */
    IWORD           off              ; /* Offset description des fichiers.   */
    IWORD           ofd              ; /* Offset donnees     des fichiers.   */
    IWORD           tot              ; /* Nombre total d'octets sur disque.  */
    IWORD           fre              ; /* Nombre d'octets libres sur disque. */
    IWORD           mxs              ; /* Taille maximum d'un fichier.       */
    IWORD           rw               ; /* Flag d'ecriture possible.          */
    IWORD           bfs              ; /* Numero bloc source pour copie.     */
    IWORD           bfd              ; /* Numero bloc dest   pour copie.     */
    IWORD           nfs              ; /* Numero fic source  pour copie.     */
    IWORD           nfd              ; /* Numero fic dest    pour copie.     */
    IWORD           ids              ; /* Identifiant du bloc spare.         */
    IWORD           bsy              ; /* Traitement en cours OUI/NON.       */
    IWORD           fidc             ; /* Id fichier en cours de creation.   */ 
    IWORD           fidt             ; /* Id fichier en cours de traitement. */
  END               Fdf_disk         ; /* ---------------------------------- */


/*
    3    Public variables
    _____________________
*/


/*
    4    Prototypes of public procedures
    ____________________________________
*/


#endif                                 /* End of FILE_H.                     */
