/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS                       File "include/tim_stm32.h"           */
/*  Constants and definitions for low level STM32 timers.                    */
/*                                                                           */
/*  Author : JYA                        Last modification : 01/23/2019       */
/*                                                                           */
/*  (c) Copyright Hyperpanel Lab, All Rights Reserved                        */
/*  IDDN.FR.001.320001.*.S.P.1990.000.10000                                  */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations ========================================================


         1  Defines
         2  Structures and typedefs
         3  Public variables
         4  Prototypes of public procedures


*/


#ifndef TIM_STM32_H                    /* If this file is not already        */
#define TIM_STM32_H                    /* included, include it.              */




/*
    1    Defines
    ____________
*/



/*
    2    Structures and typedefs
    ____________________________
*/

typedef struct                         /* Low-level TIM context structure    */
  BEGIN                                /* ---------------------------------- */
    CHAR           *uname            ; /* Timer user defined name            */
    UCHAR          ldev              ; /* ST defined timer number            */
    UCHAR          chan              ; /* Optionnal channel number           */
  END              Ref_tim32         ; /* ---------------------------------- */


typedef struct                         /* Low-level TIM context structure    */
  BEGIN                                /* ---------------------------------- */
    UCHAR          state             ; /* b0  0:Not init      1:Initialized  */
                                       /* b1  0:No call-back  1:Call-back    */
                                       /* b2  0:Stopped       1:Running      */
                                       /* b3  0:IT disabled   1:IT enabled   */
    IWORD          mode              ; /* 0:One time 1:Clock mode            */
    IWORD          (*cb)()           ; /* Call-back procedure                */
    INT            cbpar             ; /* Call-back procedure parameter      */
  END              Ctx_tim32         ; /* ---------------------------------- */


/*
    3    Public variables
    _____________________
*/

extern Ctx_tim32 ctx_tim32[]         ; /* Context table of tim_stm32.c       */
extern const Sdrv_desc sdrv_flt      ; /* FILTER system drv interface        */


/*
    4    Prototypes of public procedures
    ____________________________________
*/

/* Declaration of the procedures ........................................... */

IWORD  init_tim_stm32         (IWORD                                         );
IWORD  end_tim_stm32          (IWORD                                         );
IWORD  int_ctrl_tim_stm32     (IWORD, IWORD(*)(), IWORD                      );
IWORD  start_tim_stm32        (IWORD, UINT, IWORD                            );
UINT   get_tim_stm32          (IWORD                                         );
IWORD  get_ref_tim_stm32      (CHAR*, IWORD*, IWORD*                         );
IWORD  get_res_tim_stm32      (IWORD, UINT*, UINT*                           );
IWORD  get_sta_tim_stm32      (IWORD                                         );
IWORD  handlerTIM_stm32       (IWORD, IWORD                                  );
IWORD  handlerRCU_stm32       (IWORD, IWORD                                  );
IWORD  handlerSND_stm32       (IWORD, IWORD                                  );
IWORD  filter_stm32           (volatile UINT*, UINT, IWORD, IWORD, 
                               IWORD(*)(), IWORD                             );
IWORD  handlerFLT_stm32       (IWORD, IWORD                                  );





#endif                                 /* End of NTIM_STM32_H                */
